#include "main.h"
#include "monetra.h"

#define if_chkvar(var) if ((var != NULL) && strlen(var))

M_uintptr M_SPEC M_Ping(M_CONN *conn)
{
	M_uintptr identifier;

	identifier = M_TransNew(conn);
	M_TransKeyVal(conn, identifier, "action", "ping");
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

#ifndef LIBMONETRA_SPEC_STDCALL
/* TRANSACTION ROUTINES */
M_uintptr M_SPEC M_Sale(
	M_CONN *conn,
	char *username,
	char *password,
	char *trackdata,
	char *account,
	char *expdate,
	double amount,
	char *street,
	char *zip,
	char *cv,
	char *comments,
	char *clerkid,
	char *stationid,
	long ptrannum
) {
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		((trackdata == NULL) && ((account == NULL) || (expdate == NULL))) ||
		(amount <= 0)
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_SALE);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(trackdata) M_TransParam(conn, identifier, MC_TRACKDATA, trackdata);
	if_chkvar(account) M_TransParam(conn, identifier, MC_ACCOUNT, account);
	if_chkvar(expdate) M_TransParam(conn, identifier, MC_EXPDATE, expdate);
	if_chkvar(street) M_TransParam(conn, identifier, MC_STREET, street);
	if_chkvar(zip) M_TransParam(conn, identifier, MC_ZIP, zip);
	if_chkvar(cv) M_TransParam(conn, identifier, MC_CV, cv);
	if (amount > 0.00) {
		M_TransParam(conn, identifier, MC_AMOUNT, amount);
	}

	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_PreAuth(
	M_CONN *conn,
	char *username,
	char *password,
	char *trackdata,
	char *account,
	char *expdate,
	double amount,
	char *street,
	char *zip,
	char *cv,
	char *comments,
	char *clerkid,
	char *stationid,
	long ptrannum
) {
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		((trackdata == NULL) && ((account == NULL) || (expdate == NULL))) ||
		(amount <= 0)
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_PREAUTH);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(trackdata) M_TransParam(conn, identifier, MC_TRACKDATA, trackdata);
	if_chkvar(account) M_TransParam(conn, identifier, MC_ACCOUNT, account);
	if_chkvar(expdate) M_TransParam(conn, identifier, MC_EXPDATE, expdate);
	if_chkvar(street) M_TransParam(conn, identifier, MC_STREET, street);
	if_chkvar(zip) M_TransParam(conn, identifier, MC_ZIP, zip);
	if_chkvar(cv) M_TransParam(conn, identifier, MC_CV, cv);
	if (amount > 0.00) {
		M_TransParam(conn, identifier, MC_AMOUNT, amount);
	}

	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Void(M_CONN *conn, char *username, char *password, M_int64 ttid, long ptrannum)
{
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		((ttid == -1) && (ptrannum == -1))
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_VOID);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);

	if (ttid > 0) {
		M_TransParam(conn, identifier, MC_TTID, ttid);
	} else if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_PreAuthCompletion(M_CONN *conn, char *username, char *password,
                                     double finalamount, M_int64 ttid, long ptrannum)
{
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		(finalamount <= 0) ||
		((ttid == -1) && (ptrannum == -1))
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_PREAUTHCOMPLETE);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	M_TransParam(conn, identifier, MC_AMOUNT, finalamount);
	if (ttid > 0) {
		M_TransParam(conn, identifier, MC_TTID, ttid);
	} else if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Force(
	M_CONN *conn,
	char *username,
	char *password,
	char *trackdata,
	char *account,
	char *expdate,
	double amount,
	char *authcode,
	char *comments,
	char *clerkid,
	char *stationid,
	long ptrannum
) {
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		((trackdata == NULL) && ((account == NULL) || (expdate == NULL))) ||
		(amount <= 0)
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_FORCE);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(trackdata) M_TransParam(conn, identifier, MC_TRACKDATA, trackdata);
	if_chkvar(account) M_TransParam(conn, identifier, MC_ACCOUNT, account);
	if_chkvar(expdate) M_TransParam(conn, identifier, MC_EXPDATE, expdate);
	if (amount > 0.00) {
		M_TransParam(conn, identifier, MC_AMOUNT, amount);
	}

	if_chkvar(authcode) M_TransParam(conn, identifier, MC_APPRCODE, authcode);
	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Return(
	M_CONN *conn,
	char *username,
	char *password,
	char *trackdata,
	char *account,
	char *expdate,
	double amount,
	char *comments,
	char *clerkid,
	char *stationid,
	long ptrannum
) {
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		((trackdata == NULL) && ((account == NULL) || (expdate == NULL))) ||
		(amount <= 0)
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_RETURN);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(trackdata) M_TransParam(conn, identifier, MC_TRACKDATA, trackdata);
	if_chkvar(account) M_TransParam(conn, identifier, MC_ACCOUNT, account);
	if_chkvar(expdate) M_TransParam(conn, identifier, MC_EXPDATE, expdate);
	if (amount > 0.00) {
		M_TransParam(conn, identifier, MC_AMOUNT, amount);
	}

	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Override(
	M_CONN *conn,
	char *username,
	char *password,
	char *trackdata,
	char *account,
	char *expdate,
	double amount,
	char *street,
	char *zip,
	char *cv,
	char *comments,
	char *clerkid,
	char *stationid,
	long ptrannum
) {
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		((trackdata == NULL) && ((account == NULL) || (expdate == NULL))) ||
		(amount <= 0)
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_OVERRIDE);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(trackdata) M_TransParam(conn, identifier, MC_TRACKDATA, trackdata);
	if_chkvar(account) M_TransParam(conn, identifier, MC_ACCOUNT, account);
	if_chkvar(expdate) M_TransParam(conn, identifier, MC_EXPDATE, expdate);
	if_chkvar(street) M_TransParam(conn, identifier, MC_STREET, street);
	if_chkvar(zip) M_TransParam(conn, identifier, MC_ZIP, zip);
	if_chkvar(cv) M_TransParam(conn, identifier, MC_CV, cv);
	if (amount > 0.00) {
		M_TransParam(conn, identifier, MC_AMOUNT, amount);
	}

	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Settle(M_CONN *conn, char *username, char *password, char *batch)
{
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_SETTLE);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if ((batch == NULL) || *batch == '\0') {
		M_TransParam(conn, identifier, MC_BATCH, "all");
	} else {
		M_TransParam(conn, identifier, MC_BATCH, batch);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

/* USER-LEVEL ADMINISTRATIVE REQUESTS */

M_uintptr M_SPEC M_Gut(
	M_CONN *conn,
	char *username,
	char *password,
	int type,
	char *myacct,
	char *clerkid,
	char *stationid,
	char *comments,
	double ptrannum,
	char *bdate,
	char *edate
) {
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
	M_TransParam(conn, identifier, MC_ADMIN, MC_ADMIN_GUT);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if (type > 0) {
		M_TransParam(conn, identifier, MC_AUDITTYPE, type);
	}

	if_chkvar(myacct) M_TransParam(conn, identifier, MC_ACCT, myacct);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(bdate) M_TransParam(conn, identifier, MC_BDATE, bdate);
	if_chkvar(edate) M_TransParam(conn, identifier, MC_EDATE, edate);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Gl(
	M_CONN *conn,
	char *username,
	char *password,
	int type,
	char *myacct,
	char *batch,
	char *clerkid,
	char *stationid,
	char *comments,
	double ptrannum,
	char *bdate,
	char *edate
) {
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
	M_TransParam(conn, identifier, MC_ADMIN, MC_ADMIN_GL);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if (type > 0) {
		M_TransParam(conn, identifier, MC_AUDITTYPE, type);
	}

	if_chkvar(myacct) M_TransParam(conn, identifier, MC_ACCT, myacct);
	if_chkvar(batch) M_TransParam(conn, identifier, MC_BATCH, batch);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(bdate) M_TransParam(conn, identifier, MC_BDATE, bdate);
	if_chkvar(edate) M_TransParam(conn, identifier, MC_EDATE, edate);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Gft(
	M_CONN *conn,
	char *username,
	char *password,
	int type,
	char *myacct,
	char *clerkid,
	char *stationid,
	char *comments,
	double ptrannum,
	char *bdate,
	char *edate
) {
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
	M_TransParam(conn, identifier, MC_ADMIN, MC_ADMIN_GFT);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if (type > 0) {
		M_TransParam(conn, identifier, MC_AUDITTYPE, type);
	}

	if_chkvar(myacct) M_TransParam(conn, identifier, MC_ACCT, myacct);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(bdate) M_TransParam(conn, identifier, MC_BDATE, bdate);
	if_chkvar(edate) M_TransParam(conn, identifier, MC_EDATE, edate);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

/*
 * long M_Ub(M_CONN *conn, char *username, char *password)
 * {
 * _M_CONN *conn=(_M_CONN *)(*conn);
 * M_TRANS trans;
 * M_ClearTransaction(&trans);
 * if (username == NULL || password == NULL)
 *  return(M_ERROR);
 * trans.trans_type=M_UB;
 * trans.username=username;
 * trans.password=password;
 * return(M_SendTransaction(conn, trans));
 * }
 */

M_uintptr M_SPEC M_Rs(M_CONN *conn, char *username, char *password, char *stationid)
{
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL) || (stationid == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
	M_TransParam(conn, identifier, MC_ADMIN, MC_ADMIN_RS);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Qc(
	M_CONN *conn,
	char *username,
	char *password,
	char *clerkid,
	char *stationid,
	char *comments,
	double ptrannum
) {
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
	M_TransParam(conn, identifier, MC_ADMIN, MC_ADMIN_QC);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Chkpwd(M_CONN *conn, char *username, char *password)
{
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_CHKPWD);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Bt(M_CONN *conn, char *username, char *password)
{
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
	M_TransParam(conn, identifier, MC_ADMIN, MC_ADMIN_BT);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

/* ENGINE-ADMIN REQUESTS */

M_uintptr M_SPEC M_Chngpwd(M_CONN *conn, char *adminpassword, char *newpassword)
{
	M_uintptr identifier;

	if ((adminpassword == NULL) || (newpassword == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_CHNGPWD);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpassword);
	if_chkvar(newpassword) M_TransParam(conn, identifier, MC_PWD, newpassword);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_ListUsers(M_CONN *conn, char *password)
{
	M_uintptr identifier;

	if (password == NULL) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_LISTUSERS);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_EnableUser(M_CONN *conn, char *adminpwd, char *user)
{
	M_uintptr identifier;

	if ((adminpwd == NULL) || (user == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ENABLEUSER);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpwd);
	if_chkvar(user) M_TransParam(conn, identifier, MC_USER, user);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_DisableUser(M_CONN *conn, char *adminpwd, char *user)
{
	M_uintptr identifier;

	if ((adminpwd == NULL) || (user == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_DISABLEUSER);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpwd);
	if_chkvar(user) M_TransParam(conn, identifier, MC_USER, user);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_GetUserInfo(M_CONN *conn, char *adminpwd, char *user)
{
	M_uintptr identifier;

	if ((adminpwd == NULL) || (user == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_GETUSERINFO);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpwd);
	if_chkvar(user) M_TransParam(conn, identifier, MC_USER, user);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_DelUser(M_CONN *conn, char *password, char *user)
{
	M_uintptr identifier;

	if ((password == NULL) || (user == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_DELUSER);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(user) M_TransParam(conn, identifier, MC_USER, user);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_ListStats(M_CONN *conn, char *adminpwd)
{
	M_uintptr identifier;

	if (adminpwd == NULL) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_LISTSTATS);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpwd);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Export(M_CONN *conn, char *adminpwd, char *filename)
{
	M_uintptr identifier;

	if ((filename == NULL) || (adminpwd == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_EXPORT);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpwd);
	if_chkvar(filename) M_TransParam(conn, identifier, MC_FILE, filename);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Import(M_CONN *conn, char *adminpwd, char *filename)
{
	M_uintptr identifier;

	if ((filename == NULL) || (adminpwd == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_IMPORT);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpwd);
	if_chkvar(filename) M_TransParam(conn, identifier, MC_FILE, filename);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

/* LEGACY USER FUNCTIONS */

const char *M_SPEC M_GetUserParam(M_CONN *conn, M_uintptr identifier, int key)
{
	const char *str_key = NULL;

	if (!M_verify_trans_in_queue(conn, identifier)) {
		return NULL;
	}

	str_key = M_GetUserArgString(key);
	if (str_key == NULL) {
		return NULL;
	}

	return M_ResponseParam(conn, identifier, str_key);
}

#endif /* ifndef LIBMONETRA_SPEC_STDCALL */
