#include "main.h"
#include "atouint64.h"
#include "mstdlib.h"

#ifndef _WIN32
#    include <strings.h>
#endif

static char *M_encode_quoted(const char *str, int len);
static char *M_StructureTransaction(M_CONN *myconn, M_QUEUE *queue);

void UnescapeQuotes(char *quoted_data)
{
	int i, cnt;

	for (i = 0, cnt = 0; quoted_data != NULL && quoted_data[i] != 0; i++) {
		if ((quoted_data[i] == '"') && (quoted_data[i + 1] == '"')) {
			/* Skip one of the quotes */
			i++;
		}

		quoted_data[cnt++] = quoted_data[i];
	}

	if (quoted_data) {
		quoted_data[cnt] = 0;
	}
}

#define TRANS_BLOCK_SIZE 4096
#define trans_append_block(ret, real_size, alloc_size, data, len) \
        do { \
		if (real_size + len + 1 > alloc_size) { \
			alloc_size = (((real_size + len + 1) / TRANS_BLOCK_SIZE) + 1) *TRANS_BLOCK_SIZE; \
			ret = (char *)M_realloc(ret, alloc_size); \
		} \
		memcpy(ret + real_size, data, len); \
		real_size += len; \
		ret[real_size] = 0; \
	} while (0)

static char *M_StructureTransaction(M_CONN *conn, M_QUEUE *queue)
{
	char *transaction = NULL;
	char *temp = NULL;
	int alloc_len = 0;
	int len = 0, templen = 0;
	int i;

	UNUSED(conn);

	for (i = 0; i < queue->transaction_fields; i++) {
		if (
			!strcasecmp(queue->transaction[i].key,   "action") &&
			!strcasecmp(queue->transaction[i].value, "ping")
		) {
			transaction = M_malloc(7);
			M_snprintf(transaction, 7, "%s", "PING\r\n");
			return transaction;
		}
	}

	for (i = 0; i < queue->transaction_fields; i++) {
		templen = strlen(queue->transaction[i].key);
		trans_append_block(transaction, len, alloc_len, queue->transaction[i].key, templen);
		trans_append_block(transaction, len, alloc_len, "=", 1);
		if (queue->transaction[i].value) {
			templen = strlen(queue->transaction[i].value);
			temp = M_encode_quoted(queue->transaction[i].value, templen);
			if (temp == NULL) {
				trans_append_block(transaction, len, alloc_len, queue->transaction[i].value, templen);
			} else {
				templen = strlen(temp);
				trans_append_block(transaction, len, alloc_len, temp, templen);
				M_free(temp);
			}
		}

		trans_append_block(transaction, len, alloc_len, "\r\n", 2);
	}

	return transaction;
}

int M_SendTransaction(M_CONN *conn, M_QUEUE *queue)
{
	char *transaction = NULL;
	int ret = 0;

	transaction = M_StructureTransaction(conn, queue);

	if (((*conn)->conn_method == M_SOCKETS) || ((*conn)->conn_method == M_SSL)) {
		ret = M_SendTransaction_IP(conn, queue->identifier, transaction);
	}

	M_free(transaction);

	/* pos=M_AddOutstandingAuth(conn, identifier); */
	M_lock(conn);
	queue->status = M_PENDING;
	M_unlock(conn);

	return ret;
}

int M_verify_trans_in_queue(M_CONN *conn, M_uintptr identifier)
{
	void *trans = (void *)identifier;
	int ret = 0;
	M_QUEUE *queue = NULL, *ptr = NULL;

	if (conn == NULL) {
		return 0;
	}

	M_lock(conn);
	if ((*conn)->validate_identifier) {
		queue = (*conn)->queue;
		ptr = queue;
		while (ptr != NULL) {
			if (ptr == trans) {
				ret = 1;
				break;
			}

			ptr = ptr->next;
			/* Don't remember if it's circular .. but just in case */
			if (ptr == queue) {
				break;
			}
		}
	} else {
		ret = 1;
	}

	M_unlock(conn);
	return ret;
}

/* Returns NULL if no quoting needed, otherwise allocated
 * NULL-terminated buffer of quoted data */
static char *M_encode_quoted(const char *str, int len)
{
	int alloc_len = 0;
	int cnt = 0, i;
	char *ret = NULL;
	int needs_quoting = 0; /* 0 = no, 1 = yes, 2 = yes and has embedded quotes */

	for (i = 0; i < len; i++) {
		switch (str[i]) {
			case '\r':
			case '\n':
			case ',':
				needs_quoting = 1;
				break;
			case '"':
				needs_quoting = 2;
				break;
		}

		if (needs_quoting == 2) {
			break;
		}
	}

	if (!needs_quoting) {
		return NULL;
	}

	if (needs_quoting == 2) {
		alloc_len = (len * 2) + 2;
	} else if (needs_quoting == 1) {
		alloc_len = len + 2;
	}

	ret = M_malloc(alloc_len + 1);

	if (needs_quoting == 1) {
		ret[0] = '"';
		memcpy(ret + 1, str, len);
		ret[len + 1] = '"';
		ret[len + 2] = 0;
	} else {
		ret[cnt++] = '"';
		for (i = 0; i < len; i++) {
			if (str[i] == '"') {
				ret[cnt++] = '"';
			}

			ret[cnt++] = str[i];
		}

		ret[cnt++] = '"';
		ret[cnt] = 0;
	}

	return ret;
}
